/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.actions;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.ISharedImages;

import de.bsvrz.buv.plugin.anlagenstatus.export.AnlagenStatusExport;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;

/**
 * Aktion zum Exportieren des Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: $
 */
public class AnlagenStatusExportierenAktion extends Action {

	/** Viewer. */
	private final ScrollingGraphicalViewer viewer;

	/**
	 * Konstruktor.
	 * 
	 * @param viewer
	 *            der View
	 */
	public AnlagenStatusExportierenAktion(final ScrollingGraphicalViewer viewer) {
		super("Exportieren");
		this.viewer = viewer;
		setToolTipText("Exportiert den aktuellen Anlagenstatus");
		setImageDescriptor(PluginAnlagenStatus.getDefault().getImageDescriptor(
				ISharedImages.IMG_ETOOL_SAVE_EDIT));
	}

	/**
	 * {@inheritDoc}.
	 * 
	 * @see org.eclipse.jface.action.Action#runWithEvent(org.eclipse.swt.widgets.Event)
	 */
	@Override
	public void runWithEvent(final Event event) {
		final FileDialog dialog = new FileDialog(
				viewer.getControl().getShell(), SWT.SAVE);
		final String[] filterNames = new String[] { "HTML Files",
				"All Files (*)" };
		final String[] filterExtensions = new String[] { "*.html", "*" };

		// String filterPath = "/";
		// final String platform = SWT.getPlatform();
		// if ("win32".equals(platform) || "wpf".equals(platform)) {
		// filterPath = "c:\\";
		// }

		final String filterPath = System.getProperty("user.home");
		dialog.setFilterNames(filterNames);
		dialog.setFilterExtensions(filterExtensions);
		dialog.setFilterPath(filterPath);

		final Date fdate = new Date();

		dialog.setFileName(bildeDateiname(fdate));

		// final String filename = dialog.open();
		final String filename = dateiAuswaehlen(dialog);
		if (filename != null && filename.length() > 0) {
			final AnlagenStatusExport exp = new AnlagenStatusExport(new File(
					filename), fdate, viewer);
			exp.run();
		}
	}

	/**
	 * Erzeugt den Dateinamen f&uumlr den Export.
	 * 
	 * @param date
	 *            Datum des Exportes
	 * @return Dateiname
	 */
	private String bildeDateiname(final Date date) {
		final DateFormat df = new SimpleDateFormat("dd_MM_yyyy_HHmm");

		return "Anlagenstatus_" + df.format(date) + ".html";
	}

	/**
	 * Auswahl einer Datei mit &Uuml;berpr&uuml;fung, ob diese bereits
	 * existiert.
	 * 
	 * @param dialog
	 *            FileDialog
	 * @return Dateiname
	 */
	private String dateiAuswaehlen(final FileDialog dialog) {
		String fileName = null;
		boolean done = false;

		while (!done) {
			fileName = dialog.open();
			if (fileName == null) {
				// cancel
				done = true;
			} else {
				if (checkFileExists(fileName)) {
					done = true;
				}
			}
		}

		return fileName;
	}

	/**
	 * berprft, ob die zu schreibende Datei bereits existiert. Wenn ja, wird
	 * der Nutzer gefragt, ob sie berschrieben werden darf.
	 * 
	 * @param display
	 *            Display fr den eventuellen Messagedialog
	 * @param newFilePath
	 *            Pfad zur Datei
	 * @return true - Schreiben erlaubt
	 */
	private boolean checkFileExists(final String newFilePath) {
		boolean ok = true;
		final File f = new File(newFilePath);
		if (f.exists()) {
			// berschreiben??
			ok = MessageDialog
					.openQuestion(
							viewer.getControl().getShell(),
							"Die Datei '" + newFilePath + "' existiert bereits",
							"Die Datei '"
									+ newFilePath
									+ "' existiert bereits. Mchten Sie diese berschreiben?");
			if (ok) {
				f.delete();
			}
		}
		return ok;
	}
}
