/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.actions;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.swt.widgets.Event;

import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusRootEditPart;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Aktion zum Ausblenden der untergeordneten Elemente eines Knotens.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: $
 */
public class KnotenUntergeordneteElementeAusblendenAktion extends Action {

	/** Der selektierte EditPart. */
	private final AnlagenStatusEditPart part;

	/** Debug-Logger. */
	private final Debug log = Debug.getLogger();

	/**
	 * Konstruktor.
	 * 
	 * @param part
	 *            EditPart
	 */
	public KnotenUntergeordneteElementeAusblendenAktion(
			final AnlagenStatusEditPart part) {
		super("Untergeordnete Elemente ausblenden");
		setToolTipText("Blendet die untergordneten Elemente aus");
		this.part = part;
	}

	/** {@inheritDoc}. */
	@Override
	public void runWithEvent(final Event event) {
		log.finer("Untergeordnete Elemente ausblenden: " + (part.getModel()));
		final EditPartViewer viewer = part.getViewer();
		double zoom = ((AnlagenStatusRootEditPart) viewer.getRootEditPart())
				.getZoomManager().getZoom();

		((AnlagenStatusRootEditPart) viewer.getRootEditPart()).getZoomManager()
				.setZoom(1.0);

		((AnlagenStatusKnoten) part.getModel()).untergeordneteEbeneAusblenden();

		((AnlagenStatusRootEditPart) viewer.getRootEditPart())
				.bildNeuBerechnen();

		((AnlagenStatusRootEditPart) viewer.getRootEditPart()).getZoomManager()
				.setZoom(zoom);
	}
}
