/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.einstellungen;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;

import com.bitctrl.lib.eclipse.resources.WidgetColorRegistry;

/**
 * Hilfreiche Methoden fr die Arbeit mit Farben.
 * 
 * @author BitCtrl Systems GmbH, Schumann
 * @version $Id: ColorUtilities.java 23258 2010-05-21 14:44:40Z gieseler $
 */
public final class ColorUtilities {

	/**
	 * Konvertiert RGB-Wert nach long.
	 * 
	 * @param rgb
	 *            ein RGB-Wert.
	 * @return 8bit rot, 8bit grn und 8bit blau.
	 */
	public static long rgbToLong(final RGB rgb) {
		final long red = (long) rgb.red << 16;
		final long green = (long) rgb.green << 8;
		final long blue = rgb.blue;
		return red + green + blue;
	}

	/**
	 * Konvertiert einen long-Wert in einen RGB-Wert.
	 * 
	 * @param rgb
	 *            ein long-Wert.
	 * @return das RGB quivalent.
	 */
	public static RGB longToRgb(final long rgb) {
		final int red = (int) rgb >> 16;
		final int green = (int) rgb >> 8 & 255;
		final int blue = (int) rgb & 255;
		return new RGB(red, green, blue);
	}

	/**
	 * Konstruktor verstecken.
	 */
	private ColorUtilities() {
		// nix
	}

	/**
	 * Gibt eine Color zu einem Long-Wert zur&uuml;ck.
	 * 
	 * @param longcolor
	 *            Farbwert
	 * @return Color
	 */
	public static Color longToColor(final long longcolor) {
		return WidgetColorRegistry.getInstance().getColor(
				ColorUtilities.longToRgb(longcolor));
	}

}
