/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.export;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusTopEditPart;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zum Export des Anlagenstatus als HTML-Datei.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusExport.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public class AnlagenStatusExport {

	/** DebugLogger fr Debug-Ausgaben. */
	private static final Debug DEBUG = Debug.getLogger();

	/**
	 * Export-Datei.
	 */
	private final File exportFile;

	/**
	 * Ausgabe-Stream.
	 */
	private BufferedOutputStream bos;

	/**
	 * Datum des Exports.
	 */
	private final Date exportDatum;

	/**
	 * Der Viewer.
	 */
	private final ScrollingGraphicalViewer viewer;

	/**
	 * Konstruktor.
	 * 
	 * @param exportfile
	 *            Export-Datei
	 * @param date
	 *            Datum des Exports
	 * @param viewer
	 *            Viewer
	 */
	public AnlagenStatusExport(final File exportfile, final Date date,
			final ScrollingGraphicalViewer viewer) {
		this.exportFile = exportfile;
		exportDatum = date;
		this.viewer = viewer;
		try {
			bos = new BufferedOutputStream(new FileOutputStream(exportFile));
		} catch (final IOException e) {
			DEBUG.error("Fehler beim anlegen der Datei: "
					+ exportFile.getAbsoluteFile());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	// ------------------------------------------------------------------------
	// SONSTIGE METHODEN
	// ------------------------------------------------------------------------

	/** Konvertiert die Daten in eine Konfigurationsdatei. */
	public void run() {
		StringBuffer sb = new StringBuffer();

		// Kopfbereich anlegen
		final String ueberschrift = erzeugeUeberschrift();

		sb.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
		sb.append("<html><head>\n");
		sb.append("<meta http-equiv=\"content-type\" content=\"text/html; charset=ISO-8859-1\">\n");
		sb.append("<title>" + ueberschrift + "</title>\n");
		sb.append("<style type=\"text/css\">\n");

		for (HtmlStyles style : HtmlStyles.values()) {
			final String sd = style.getStyleDefinition();
			if (sd != null && sd.length() > 0) {
				sb.append(" " + style.getStyleDefinition() + "\n");
			}
		}
		// sb
		//
		// .append("tr { padding:0px; margin:0px; text-align:center; white-space:nowrap; }\n");
		// sb
		//
		// .append("td { padding:0px; margin:0px; text-align:left; white-space:nowrap; }\n");

		// .b { color:#000000; font-family:Courier New,Courier; font-size:15px;
		// }
		// .w { color:#FFFFFF; font-family:Courier New,Courier; font-size:15px;
		// }
		// ))</style>
		sb.append("</style>");
		sb.append("</head>\n");
		sb.append("<body>\n");

		sb.append("<H2>" + ueberschrift + "</H2>\n");
		schreibeInDatei(sb);

		schreibeObjekte();

		sb = new StringBuffer();
		sb.append("</body>\n");
		sb.append("</html>\n");

		schreibeInDatei(sb);

		try {
			bos.close();
		} catch (final IOException e) {
			DEBUG.error("Fehler beim Schlieen der Datei.");
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/**
	 * Schreibt die Objekte der Ansicht in die Datei.
	 * <p>
	 * Die Anordnung erfolgt mit Hierarchierichtung von links nach rechts,
	 * zeilenweise f&uuml;r die oberste Hierarchieebene.
	 */
	private void schreibeObjekte() {
		final AnlagenStatusTopEditPart tep = (AnlagenStatusTopEditPart) viewer
				.getContents();
		final List<List<AnlagenStatusEditPart>> levelliste = new ArrayList<List<AnlagenStatusEditPart>>();

		int lines = 0;
		final int rows = tep.getLevelMap().size();

		for (int i = 0; i < tep.getLevelMap().size(); i++) {
			final List<AnlagenStatusEditPart> list = tep.getLevelMap().get(i);

			if (i == 0) {
				lines = Math.max(lines, list.size());
			}

			levelliste.add(i, list);
		}

		StringBuffer sb = new StringBuffer();
		sb.append("<table border=\"0\" rules=\"none\" cellspacing=\"30\">\n");
		schreibeInDatei(sb);

		// int line = 0;
		for (AnlagenStatusEditPart aep : levelliste.get(0)) {

			// Map<Integer, List<String>> hlist = new HashMap<Integer,
			// List<String>>();
			//
			// String htmlText = aep.getHtmlText(rows);
			// List<String> tlist = new ArrayList<String>();
			// tlist.add(htmlText);
			// hlist.put(0, tlist);
			//
			// getChildHtml(1, hlist, aep);

			sb = new StringBuffer();
			sb.append("<tr>\n");

			// sb.append("<td align=\"left\">\n");
			sb.append(aep.getHtmlText(0, rows, true));
			// sb.append("</td>\n");

			// for (int row = 1; row < rows; row++) {
			// sb.append("<td align=\"left\">\n");
			// final List<AnlagenStatusEditPart> list = levelliste.get(row);
			// if (list.size() >= line + 1) {
			// sb.append(list.get(line).getHtmlText());
			// } else {
			// sb.append("</td>\n");
			// }
			// }

			sb.append("</tr>\n");
			schreibeInDatei(sb);
			// line++;
		}
		// for (int row = 0; row < rows; row++) {
		// sb = new StringBuffer();
		// sb.append("<td align=\"center\" valign=\"top\">\n");
		// sb
		// .append("<table border=\"0\" rules=\"none\" cellspacing=\"25\">\n");
		// final List<AnlagenStatusEditPart> list = tep.getLevelMap().get(row);
		//
		// for (int line = 0; line < list.size(); line++) {
		// sb.append("<tr>\n");
		// sb.append("<td align=\"left\">\n");
		// sb.append(list.get(line).getHtmlText());
		// sb.append("</td>\n");
		// sb.append("</tr>\n");
		// }
		// sb.append("</table>\n");
		// sb.append("</td>\n");
		// schreibeInDatei(sb);
		// }

		sb = new StringBuffer();
		sb.append("</table>\n");
		schreibeInDatei(sb);
	}

	// /**
	// * @param level
	// * @param map
	// * @param part
	// */
	// void getChildHtml(final int level, final Map<Integer, List<String>> map,
	// final AnlagenStatusEditPart part) {
	// for (Object o : part.getSourceConnections()) {
	// final AnlagenStatusConnectionEditPart cep =
	// (AnlagenStatusConnectionEditPart) o;
	// final AnlagenStatusEditPart ep = (AnlagenStatusEditPart) cep
	// .getTarget();
	// final String htmlText = ep.getHtmlText(0, 0, true);
	// List<String> tlist = map.get(level);
	//
	// if (tlist == null) {
	// tlist = new ArrayList<String>();
	// }
	// tlist.add(htmlText);
	// map.put(level, tlist);
	//
	// getChildHtml(level + 1, map, ep);
	// }
	// }

	/**
	 * Schreibt einen Stringbuffer in Datei.
	 * 
	 * @param sb
	 *            Zu schreibender StringBuffer.
	 */
	private void schreibeInDatei(final StringBuffer sb) {
		try {
			bos.write(sb.toString().getBytes());
		} catch (final IOException e) {
			DEBUG.error("Fehler beim Schreiben auf Datei: "
					+ exportFile.getAbsoluteFile());
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/**
	 * Erzeugt die &Uuml;berschrift f&uuml;r die Seite.
	 * 
	 * @return &Uuml;berschrift
	 */
	private String erzeugeUeberschrift() {
		final DateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm");

		return "Anlagenstatus " + df.format(exportDatum) + "Uhr";
	}
}
