/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.figures;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Figure zur Anzeige der Erg&auml;zenden De-Fehler der FG1.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DeErgaenzendeFehlermeldungFigureFG1.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public class DeErgaenzendeFehlermeldungFigureFG1 extends
		DeErgaenzendeFehlermeldungFigure {

	// private final DeLve de;

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Part
	 */
	public DeErgaenzendeFehlermeldungFigureFG1(
			final AnlagenStatusEditPart parentpart) {
		super(parentpart);
		setLabel("Ergnzende DE-Fehlermeldung:");

		// de = (DeLve) ((AnlagenStatusKnoten) parentpart.getModel())
		// .getSystemObjekt();

		final AttributeGroup atg = DaVVerbindung.getInstanz()
				.getDaVVerbindung().getDataModel().getAttributeGroup(
						"atg.tlsLveErgnzendeDeFehlerMeldung");
		final Aspect asp = DaVVerbindung.getInstanz().getDaVVerbindung()
				.getDataModel().getAspect("asp.tlsAntwort");
		setDatadescription(new DataDescription(atg, asp));

	}

	/** {@inheritDoc} */
	@Override
	public String convertByte2(final int tlsbyte) {
		final StringBuilder result = new StringBuilder();
		// Byte 2
		for (int i = 0; i < 8; i++) {
			String addMsg = null;

			if ((tlsbyte & (1 << i)) != 0) {

				switch (i) {
				case 0:
					addMsg = "Subbus ausgefallen"; //$NON-NLS-1$
					break;
				case 1:
					addMsg = "Subbus RNR"; //$NON-NLS-1$
					break;
				case 2:
					addMsg = "Detektor gestrt"; //$NON-NLS-1$
					break;
				case 3:
					addMsg = "Auswertung gestrt"; //$NON-NLS-1$
					break;
				case 4:
					addMsg = "Schleife 1 defekt"; //$NON-NLS-1$
					break;
				case 5:
					addMsg = "Schleife 2 defekt"; //$NON-NLS-1$
					break;
				case 6:
					addMsg = "Abgleich Schleife 1"; //$NON-NLS-1$
					break;
				case 7:
					addMsg = "Abgleich Schleife 2"; //$NON-NLS-1$
					break;

				default:
					break;
				}

				if (addMsg != null) {
					if (result.length() > 0) {
						result.append(", ");
					}
					result.append(addMsg);
				}
			}
		}
		if (result.toString().length() == 0) {
			return ("keine Fehler");
		}

		return result.toString();
	}

}
