/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.internal;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.EventListenerList;

import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Definition der Funktionen, f&uuml;r die Berechtigungen innerhalb des
 * Rahmenwerkes festgelegt werden k&ouml;nnen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, Steffen Gieseler
 * 
 * @version $Id: AnlagenStatusPluginFunktion.java 23300 2010-05-25 16:13:37Z gieseler $
 */
public enum AnlagenStatusPluginFunktion {

	/** Aktion Ansicht 'Anlagenstatus' &ouml;ffnen. */
	ANLAGENSTATUS_VIEW_OEFFNEN("anlagenStatusViewOeffnen");

	/**
	 * liefert die Funktion, die mit der bergebenen Id definiert ist. Wenn
	 * keine entsprechende Funktion gefunden werden konnte liefert die Funktion
	 * den Wert <code>null</code>.
	 * 
	 * @param id
	 *            die Id
	 * @return die Funktion oder <code>null</code>
	 */
	public static AnlagenStatusPluginFunktion getFunktion(final String id) {
		AnlagenStatusPluginFunktion result = null;

		for (AnlagenStatusPluginFunktion item : AnlagenStatusPluginFunktion
				.values()) {
			if (item.funktion.getFunktionsName().equals(id)) {
				result = item;
				break;
			}
		}

		return result;
	}

	/** definiert den Freigabezustand der Funktion. */
	private boolean freigegeben;

	/** die zugeordnete Rahmenwerks-Oberflchenfunktion. */
	private OberflaechenFunktion funktion;

	/**
	 * die Liste der zu benachrichtigenden Listener, wenn sich der
	 * Freigabezustand der Funktion ndert.
	 */
	private final EventListenerList listeners = new EventListenerList();

	/**
	 * Konstruktor, erzeugt eine Funktionsdefinition basierend auf dem
	 * bergebenen Name.
	 * 
	 * Innerhalb des Rahmenwerks wird die Funktion mit der Dialog-ID, die dem
	 * Name des {@link PluginBenutzerVew} entspricht und dem Klassenname der
	 * {@link AnlagenStatusPluginFunktion} erweitert um den bergebenen Name
	 * registriert.
	 * 
	 * @param name
	 *            der Name
	 */
	private AnlagenStatusPluginFunktion(final String name) {
		freigegeben = false;
		funktion = new OberflaechenFunktion(AnlagenStatusPluginFunktion.class
				.getName()
				+ "." + name,
				AnlagenStatusBerechtigungsVerwaltung.getInstanz(),
				PluginAnlagenStatus.class.getName(), name);
		OberflaechenBerechtigung.getInstanz().hinzufuegenOberflaechenFunktion(
				funktion);
	}

	/**
	 * fgt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion hinzu.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void addFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.add(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * liefert den Freigabezustand der Funktion in der aktuellen Umgebung.
	 * 
	 * @return den Zustand
	 */
	public boolean isFreigegeben() {
		if (!freigegeben) {
			final List<SystemObject> objekt = new ArrayList<SystemObject>();
			objekt.add(DaVVerbindung.getInstanz().getDaVVerbindung()
					.getLocalApplicationObject());
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(objekt,
							PluginAnlagenStatus.class.getName());
		}

		return freigegeben;
	}

	/**
	 * entfernt einen Listener zur Beobachtung des Berechtigungszustands der
	 * Funktion.
	 * 
	 * @param listener
	 *            der Listener
	 */
	public void removeFunktionsBerechtigungsListener(
			final FunktionsBerechtigungsListener listener) {
		listeners.remove(FunktionsBerechtigungsListener.class, listener);
	}

	/**
	 * setzt den Freigabezustand der Funktion.
	 * 
	 * @param freigegeben
	 *            der Zustand
	 */
	void setFreigegeben(final boolean freigegeben) {
		if (freigegeben != this.freigegeben) {
			this.freigegeben = freigegeben;
			for (FunktionsBerechtigungsListener listener : listeners
					.getListeners(FunktionsBerechtigungsListener.class)) {
				listener.berechtigungsAenderung();
			}
		}
	}
}
