/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.AnlagenStatusFunktionsgruppe;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsSysKnotenNummer;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;

/**
 * Modellobjekt f&uuml;r EAK.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusEakKnoten.java 33710 2012-04-20 11:06:38Z gieseler
 *          $
 */
public class AnlagenStatusEakKnoten extends AnlagenStatusKnoten {

	/**
	 * Konstruktor.
	 * 
	 * @param modell
	 *            Modell
	 * @param parent
	 *            Parent
	 * @param systemObjekt
	 *            SystemObjekt
	 * @param level
	 *            Level
	 */
	public AnlagenStatusEakKnoten(final AnlagenStatusModell modell,
			final AnlagenStatusKnoten parent, final SystemObjekt systemObjekt,
			final int level) {
		super(modell, parent, systemObjekt, level);
		setName("EAK");
	}

	/** {@inheritDoc} */
	@Override
	public void erzeugeKinder() {
		final Eak eak = (Eak) getSystemObjekt();

		// DE nach DE-Kanal sortieren
		final List<De> deliste = new ArrayList<De>(eak.getDe());
		Collections.sort(deliste, new Comparator<Object>() {
			public int compare(final Object o1, final Object o2) {
				int comp = ((De) o1).getKdDe().getDatum().getDEKanal()
						.compareTo(((De) o2).getKdDe().getDatum().getDEKanal());

				// bei gleichem DE-Kanal sortiere zustzlich nach FG
				if (comp == 0) {
					AnlagenStatusFunktionsgruppe fgo1 = AnlagenStatusFunktionsgruppe
							.bestimmeFG(((De) o1));
					AnlagenStatusFunktionsgruppe fgo2 = AnlagenStatusFunktionsgruppe
							.bestimmeFG(((De) o2));
					return Integer.valueOf(fgo1.ordinal()).compareTo(
							Integer.valueOf(fgo2.ordinal()));
				}

				return comp;
			}
		});

		for (De de : deliste) {
			final AnlagenStatusDeKnoten tk = new AnlagenStatusDeKnoten(
					getAnlagenStatusModell(), this, de, getlevel() + 1);
			addKind(tk);

			getAnlagenStatusModell().addNode(tk);
		}
	}

	/** {@inheritDoc} */
	@Override
	public String getLabel() {
		return "EAK"
				+ ((Eak) getSystemObjekt()).getKdEak().getDatum()
						.getOSI2Adresse();
	}

	/** {@inheritDoc} */
	@Override
	AttTlsSysKnotenNummer setKnotenNummer() {
		return new AttTlsSysKnotenNummer(0);
	}

	/** {@inheritDoc} */
	@Override
	protected int berechneKinderHoehe() {
		return 0;
	}

	/** {@inheritDoc} */
	@Override
	protected int berechneKinderBreite() {
		return 0;
	}
}
