/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.bsvrz.buv.plugin.dobj.util.DoTypenUtil;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsSysKnotenNummer;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.AnschlussPunkt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.AnschlussPunktKommunikationsPartner;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Kri2B;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.SteuerModul;

/**
 * Modellobjekt f&uuml;r Ger&auml;te.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusGeraetKnoten.java 33710 2012-04-20 11:06:38Z
 *          gieseler $
 */
public class AnlagenStatusGeraetKnoten extends AnlagenStatusKnoten {

	/**
	 * Konstruktor.
	 * 
	 * @param modell
	 *            Modell
	 * @param parent
	 *            Parent-Knoten
	 * @param systemObjekt
	 *            das zugeh&ouml;rige Systemobjekt
	 * @param level
	 *            Hierarchieebene im Anlagenmodell
	 */
	public AnlagenStatusGeraetKnoten(final AnlagenStatusModell modell,
			final AnlagenStatusKnoten parent, final SystemObjekt systemObjekt,
			final int level) {
		super(modell, parent, systemObjekt, level);
		if (!(systemObjekt instanceof Geraet)) {
			throw new IllegalArgumentException("Systemobjekt ist kein Gert");
		}
		modell.addNode(this);
		setDoTyp(DoTypenUtil
				.getConfiguratedDoTyp("anlagenstatus.GeraetEditPart"));
	}

	/** {@inheritDoc} */
	@Override
	public void erzeugeKinder() {
		final Set<Integer> portmap = new HashSet<Integer>();

		for (AnschlussPunkt anschlussPunkt : getAnschlusspunkte()) {

			for (AnschlussPunktKommunikationsPartner anschlussPunktKommunikationsPartner : getAnschlussPunkteKommunikationsPartner(anschlussPunkt)) {
				// System.out.println(this + "Kommunikationspartner: "
				// + anschlussPunktKommunikationsPartner);
				final Geraet kommunikationsPartner = anschlussPunktKommunikationsPartner
						.getKdAnschlussPunktKommunikationsPartner().getDatum()
						.getKommunikationsPartner();
				AnlagenStatusKnoten tk = getAnlagenStatusModell().getKnoten(
						kommunikationsPartner);

				if (tk == null) {
					if (kommunikationsPartner instanceof SteuerModul
							&& !(kommunikationsPartner instanceof Kri2B)) {
						tk = new SteuerModulKnoten(getAnlagenStatusModell(),
								this, kommunikationsPartner, getlevel() + 1);
					} else {
						tk = new AnlagenStatusGeraetKnoten(
								getAnlagenStatusModell(), this,
								kommunikationsPartner, getlevel() + 1);

					}
					addKind(tk);
				}

				final AnlagenStatusConnection connection = new AnlagenStatusConnection(
						this, anschlussPunkt, tk,
						anschlussPunktKommunikationsPartner);

				// nur 1 Label pro Port
				if (portmap.contains(connection.getSourcePort())) {
					connection.setAnfangLabel(false);
				} else {
					connection.setAnfangLabel(true);
					portmap.add(connection.getSourcePort());
				}

				connectOutput(connection);

				tk.connectInput(connection);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	AttTlsSysKnotenNummer setKnotenNummer() {
		return ((Geraet) getSystemObjekt()).getKdGeraet().getDatum()
				.getKnotenNummer();
	}

	/** {@inheritDoc} */
	@Override
	public String getName() {
		return ("Gert");
	}

	/**
	 * Gibt die Liste der Anschlusspunkte zur&uuml;ck.
	 * 
	 * @return Liste der Anschlusspunkte
	 */
	private List<AnschlussPunkt> getAnschlusspunkte() {
		// Anschlusspunkte nach Port sortieren um Kreuzungen zu vermeiden
		final Geraet geraet = ((Geraet) getSystemObjekt());
		final List<AnschlussPunkt> anschlussPunkteGeraet = new ArrayList<AnschlussPunkt>(
				geraet.getAnschlussPunkteGeraet());

		final int direction = getAnlagenStatusModell().isTopDown() ? 1 : -1;

		Collections.sort(anschlussPunkteGeraet, new Comparator<Object>() {
			public int compare(final Object o1, final Object o2) {
				return direction
						* ((AnschlussPunkt) o1)
								.getKdAnschlussPunkt()
								.getDatum()
								.getPortNummer()
								.compareTo(
										((AnschlussPunkt) o2)
												.getKdAnschlussPunkt()
												.getDatum().getPortNummer());
			}
		});
		return anschlussPunkteGeraet;
	}

	/**
	 * Gibt die Liste der AnschlussPunktKommunikationsPartner eines
	 * AnschlussPunktes zur&uuml;ck.
	 * 
	 * @param anschlussPunkt
	 *            AnschlussPunkt
	 * @return Liste der AnschlussPunktKommunikationsPartner des
	 *         AnschlussPunktes
	 */
	private List<AnschlussPunktKommunikationsPartner> getAnschlussPunkteKommunikationsPartner(
			final AnschlussPunkt anschlussPunkt) {
		final List<AnschlussPunktKommunikationsPartner> anschlussPunkteKommunikationsPartner = new ArrayList<AnschlussPunktKommunikationsPartner>();

		if (anschlussPunkt.getAnschlussPunkteKommunikationsPartner() != null) {
			anschlussPunkteKommunikationsPartner.addAll(anschlussPunkt
					.getAnschlussPunkteKommunikationsPartner());
		}

		// AnschlussPunktKommunikationsPartner nach Port sortieren
		Collections.sort(anschlussPunkteKommunikationsPartner,
				new Comparator<Object>() {
					public int compare(final Object o1, final Object o2) {
						return ((AnschlussPunktKommunikationsPartner) o1)
								.getKdAnschlussPunktKommunikationsPartner()
								.getDatum()
								.getPortNummer()
								.compareTo(
										((AnschlussPunktKommunikationsPartner) o2)
												.getKdAnschlussPunktKommunikationsPartner()
												.getDatum().getPortNummer());
					}
				});

		return anschlussPunkteKommunikationsPartner;
	}
}
