/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.dobj.util.DoTypenUtil;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.attribute.AttTlsSysKnotenNummer;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.SteuerModul;

/**
 * Modellobjekt f&uuml;r Steuermodule.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: SteuerModulKnoten.java 33710 2012-04-20 11:06:38Z gieseler $
 */
public class SteuerModulKnoten extends AnlagenStatusKnoten {

	/**
	 * Konstruktor.
	 * 
	 * @param modell
	 *            Modell
	 * @param parent
	 *            Parent-Knoten
	 * @param systemObjekt
	 *            das zugeh&ouml;rige Systemobjekt
	 * @param level
	 *            Hierarchieebene im Anlagenmodell
	 */
	public SteuerModulKnoten(final AnlagenStatusModell modell,
			final AnlagenStatusKnoten parent, final SystemObjekt systemObjekt,
			final int level) {
		super(modell, parent, systemObjekt, level);
		modell.addNode(this);
		if (PluginEinstellungen.getInstance().getEinstellungen()
				.isSteuerModuleAusblenden()) {
			ausblenden();
		}

		// "anlagenstatus.SteuerModulEditPart" muss mit ID des Extension-Points
		// bereinstimmen!
		setDoTyp(DoTypenUtil
				.getConfiguratedDoTyp("anlagenstatus.SteuerModulEditPart"));
	}

	/** {@inheritDoc} */
	@Override
	public void erzeugeKinder() {
		final SteuerModul sm = (SteuerModul) getSystemObjekt();

		// EAK nach OSI2-Adresse sortieren um Kreuzungen zu vermeiden
		final List<Eak> eakliste = new ArrayList<Eak>(sm.getEak());
		Collections.sort(eakliste, new Comparator<Object>() {
			public int compare(final Object o1, final Object o2) {
				return ((Eak) o1)
						.getKdEak()
						.getDatum()
						.getOSI2Adresse()
						.compareTo(
								((Eak) o2).getKdEak().getDatum()
										.getOSI2Adresse());
			}
		});

		for (Eak eak : eakliste) {
			final AnlagenStatusEakKnoten tk = new AnlagenStatusEakKnoten(
					getAnlagenStatusModell(), this, eak, getlevel() + 1);
			addKind(tk);

			getAnlagenStatusModell().addNode(tk);
		}

	}

	/** {@inheritDoc} */
	@Override
	AttTlsSysKnotenNummer setKnotenNummer() {
		return ((SteuerModul) getSystemObjekt()).getKdGeraet().getDatum()
				.getKnotenNummer();
	}

	/** {@inheritDoc} */
	@Override
	protected int berechneKinderHoehe() {
		return 0;
	}

	/** {@inheritDoc} */
	@Override
	protected int berechneKinderBreite() {
		return 0;
	}
}
