/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import org.eclipse.draw2d.Border;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ScalableFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.AnlagenstatusDoModel;
import de.bsvrz.buv.plugin.anlagenstatus.AnlagenstatusDoTyp;
import de.bsvrz.buv.plugin.anlagenstatus.actions.VersorgungAnzeigenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.export.HtmlTools;
import de.bsvrz.buv.plugin.anlagenstatus.figures.DatenFenster;
import de.bsvrz.buv.plugin.anlagenstatus.figures.StatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.VersorgungFigure;
import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenGeraet;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusConnection;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusConnectionAnchor;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.dobj.decorator.SkalierungMediator;
import de.bsvrz.buv.plugin.dobj.decorator.ZoomVerhaltenFixMediator;
import de.bsvrz.buv.plugin.dobj.editparts.BitCtrlDoModelEditPart;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeGroupUsage;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Abstrakter EditPart Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusEditPart.java 33712 2012-04-20 13:10:58Z schumann
 *          $
 */
public abstract class AnlagenStatusEditPart extends
		BitCtrlDoModelEditPart<SystemObjekt, IFigure> implements
		PropertyChangeListener, ILegendeBaustein {

	/** Property f&uuml;r geschlossenes Zusatzfenster. */
	private static final String ZUSATZWINDOW_CLOSED_PROPERTY = "status_closed";

	/** Property f&uuml;r ge&auml;nderten Status. */
	public static final String STATUS_CHANGED_PROPERTY = "status_changed";

	/** Offset des ersten Verbindungspunkte von der Ecke des Editparts. */
	private static final int FIRST_ANCHOR_OFFSET = 8;

	/** Legendebaustein f&uuml;r die Legende. */
	private List<ILegendeBaustein> legendenBausteine;

	/** Ressourcebundle. */
	private final ResourceBundle resourceBundle = PluginAnlagenStatus
			.getDefault().getResourceBundle();
	//
	/** Figure mit den Versorgungsinformationen. */
	private VersorgungFigure versorgungFigure;

	/** Figure mit den Statusinformationen. */
	private StatusFigure statusFigure;

	/** Der zugeordnete Knoten. */
	private AnlagenStatusKnoten knoten;

	/** Flag, ob der Editpart online ist. */
	private boolean partOnline;

	/** Flag, ob der Editpart im Anlagenstatus-View angezeigt wird. */
	private boolean isInView;

	/** Das zugeh&ouml;rige Systemobjekt. */
	private SystemObjekt systemObjekt;

	/** Flag, ob das erste Mal 'refreschVisuals' aufgerufen wird. */
	private boolean firstRefreshVisuals = true;

	/** Horizontaler Abstand zwischen den Parts. */
	public static final int HORIZONTAL_SPACE = 40;

	/** Vertikaler Abstand zwischen den Parts. */
	public static final int VERTICAL_SPACE = 40;

	/** Debug-Logger. */
	private final Debug log = Debug.getLogger();

	/** Map der Source-Connection-Anchors. */
	private final Map<AnlagenStatusConnection, ConnectionAnchor> sourceAnchorMap = new HashMap<AnlagenStatusConnection, ConnectionAnchor>();

	/** Map der Source-Port-Connection-Anchors. */
	private final Map<Integer, AnlagenStatusConnectionAnchor> sourcePortAnchorMap = new HashMap<Integer, AnlagenStatusConnectionAnchor>();

	/** Map der Target-Connection-Anchors. */
	private final Map<AnlagenStatusConnection, ConnectionAnchor> targetAnchorMap = new HashMap<AnlagenStatusConnection, ConnectionAnchor>();

	/** Map der Target-Port-Connection-Anchors. */
	private final Map<Integer, AnlagenStatusConnectionAnchor> targetPortAnchorMap = new HashMap<Integer, AnlagenStatusConnectionAnchor>();

	/**
	 * Test, ob sich der Editpart im Anlagenstatus-View befindet.
	 * 
	 * @return true, wenn im Anlagenstatus-View, sonst false
	 */
	public boolean isInView() {
		RootEditPart rootEditPart = getViewer().getRootEditPart();
		return rootEditPart instanceof AnlagenStatusRootEditPart;

		// return isInView;
	}

	/**
	 * Setzt, ob sich der Editpart im Anlagenstatus-View befindet.
	 * 
	 * @param inView
	 *            true, wenn im Anlagenstatus-View, sonst false
	 */
	public void setInView(final boolean inView) {
		this.isInView = inView;
	}

	/**
	 * Pr&uuml;ft, ob der Editpart online ist.
	 * 
	 * @return true, wenn online, sonst false
	 */
	protected boolean isPartOnline() {
		return partOnline;
	}

	/** {@inheritDoc} */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);

		final Object notifier = notification.getNotifier();
		if (notifier instanceof AnlagenstatusDoModel
				|| notifier instanceof AnlagenstatusDoTyp) {
			refreshVisuals();
		}
	}

	/**
	 * Setzt die zugeh&ouml;rige Statusfigur.
	 * 
	 * @param statusFigure
	 *            zugeh&ouml;rige Statusfigur
	 */
	private void setStatusFigure(final StatusFigure statusFigure) {
		this.statusFigure = statusFigure;
	}

	/**
	 * Gibt die zugeh&ouml;rige Statusfigur zur&uuml;ck.
	 * 
	 * @return zugeh&ouml;rige Statusfigur
	 */
	public StatusFigure getStatusFigure() {
		return statusFigure;
	}

	/**
	 * Gibt die zugeh&ouml;rige Versorgungsdatenfigur zur&uuml;ck.
	 * 
	 * @return zugeh&ouml;rige Versorgungsdatenfigur
	 */
	public VersorgungFigure getVersorgungFigure() {
		return versorgungFigure;
	}

	/**
	 * Gibt den zugeh&ouml;rigen Knoten (Modell) zur&uuml;ck.
	 * 
	 * @return AnlagenStatusKnoten
	 */
	public AnlagenStatusKnoten getKnoten() {
		if (knoten == null) {
			knoten = PluginAnlagenStatus.getDefault().getAnlagenModell()
					.getKnoten(getModel().getSystemObjekt());
		}

		return knoten;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();
	}

	/**
	 * Aktualisiert den Rahmen.
	 */
	protected void updateBorder() {
		AnlagenStatusKnotenStatus anzeigeStatus = getKnotenStatus();

		if (!partOnline) {
			anzeigeStatus = AnlagenStatusKnotenStatus.UNBEKANNT;
		}

		final Border border = getFigure().getBorder();

		if (border != null && border instanceof LineBorder) {
			((LineBorder) border).setColor(ColorUtilities
					.longToColor(PluginEinstellungen.getInstance()
							.getEinstellungen()
							.getKnotenStatusBorderColor(anzeigeStatus)));
			getFigure().repaint();
		}
	}

	/**
	 * F&uuml;hrt alle Aktionen aus, die bei einer Neuberechnung des Bildes
	 * erforderlich sind.
	 */
	protected void bildNeuBerechnen() {
		closeVersorgungWindow();
		closeStatusWindow();
		erzeugeFigureInhalt();
		refresh();
		refreshVisuals();
	}

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		log.finer(this + "refreshVisuals");
		if (systemObjekt == null
				|| systemObjekt != getModel().getSystemObjekt()) {
			systemObjekt = getModel().getSystemObjekt();
			erzeugeFigureInhalt();
		}

		updateBorder();

		// if (!firstRefreshVisuals) {
		// return;
		// }

		new SkalierungMediator(this).mediate();
		new ZoomVerhaltenFixMediator(this).mediate();

		if (isInView && partOnline) {
			if (firstRefreshVisuals) {
				setPositionUndGroesse();
				firstRefreshVisuals = false;
			}
			final Point loc = ((AnlagenStatusKnoten) getModel()).getLocation();
			final Dimension size = ((AnlagenStatusKnoten) getModel()).getSize();

			log.finest(this + " pos: " + loc + " dim: " + size);

			getFigure().setSize(size);
			final Rectangle r = new Rectangle(loc, size);
			((GraphicalEditPart) getParent()).setLayoutConstraint(this,
					getFigure(), r);

			for (Object ep : getSourceConnections()) {
				((AbstractGraphicalEditPart) ep).refresh();
			}
		}

		super.refreshVisuals();
	}

	/**
	 * Erzeugt den Inhalt der Figur.
	 */
	protected void erzeugeFigureInhalt() {
	}

	/**
	 * Bestimmt den Status des Knotens.
	 * 
	 * @return {@link AnlagenStatusKnotenStatus}
	 */
	public AnlagenStatusKnotenStatus getKnotenStatus() {
		if (getKnoten() != null) {
			return getKnoten().getKnotenStatus();
		}

		return AnlagenStatusKnotenStatus.UNBEKANNT;
	}

	/** {@inheritDoc} */
	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
		final String prop = evt.getPropertyName();
		log.finer(this + "::propertyChange() with evt.propName=" + prop);

		if ("size".equals(prop)) {
			setPositionUndGroesse();
			refreshVisuals();
		} else if ("location".equals(prop)) {
			setPositionUndGroesse();
			refreshVisuals();
		} else if ("inputs".equals(prop)) {
			refreshSourceConnections();
		} else if ("outputs".equals(prop)) {
			refreshTargetConnections();
		} else if (ZUSATZWINDOW_CLOSED_PROPERTY.equals(prop)) {
			log.finest("::propertyChange() with evt.propName=" + prop);
		} else if (STATUS_CHANGED_PROPERTY.equals(prop)) {
			refreshStatus();
		}

	}

	/**
	 * Aktualisiert den Status.
	 */
	private void refreshStatus() {
		if (firstRefreshVisuals) {
			refreshVisuals();
		} else {
			updateBorder();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void activate() {
		super.activate();

		knoten = getKnoten();
		if (knoten != null) {
			// throw new IllegalArgumentException(
			// "Modellobjekt nicht im Anlagenstatusmodell!");
			// }
			knoten.addPropertyChangeListener(this);
		}
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure getToolTip() {
		return getFigure().getToolTip();
	}

	/** {@inheritDoc} */
	@Override
	public void deactivate() {
		if (getKnoten() != null) {
			knoten.removePropertyChangeListener(this);
		}

		super.deactivate();
		// getModel().getDoTyp().eAdapters().remove(this);
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return "AnlagenStatusEditPart(" + getModel().getSystemObjekt() + ")";
	}

	/**
	 * &Ouml;ffnet das Versorgungsdatenfenster.
	 */
	public void openVersorgungWindow() {
		versorgungFigure = new VersorgungFigure(this);
		final IFigure figure2 = getFigure();
		versorgungFigure.setScale(((ScalableFigure) figure2).getScale());
	}

	/**
	 * Schlie&szlig;t das Versorgungsdatenfenster.
	 */
	public void closeVersorgungWindow() {
		if (versorgungFigure != null) {
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

				public void run() {
					if (versorgungFigure != null) {
						final IFigure parent2 = versorgungFigure.getParent();
						parent2.remove(versorgungFigure);
						versorgungFigure = null;
					}
				}

			});
		}
	}

	/**
	 * Erzeugt das Men&uuml; f&uuml;r die Versorgungsdatenanzeige.
	 * <p>
	 * Es werden alle konfigurierenden Attributgruppen zur Anzeige angeboten.
	 * </p>
	 * 
	 * @return IContributionItem
	 */
	public IContributionItem erzeugeVersorgungsDatenMenu() {
		final IMenuManager manager = new MenuManager(
				resourceBundle.getString("VersorgungAbrufen.menu"));
		final SystemObjectType type = getKnoten().getSystemObject().getType();

		for (AttributeGroup atg : type.getAttributeGroups()) {
			for (AttributeGroupUsage u : atg.getAttributeGroupUsages()) {
				// nur konfigurierende ATG
				if (u.isConfigurating()) {
					for (Object a : VersorgungAnzeigenAktion
							.erzeugeMenuAktionen(this, atg, u.getAspect())) {

						if (a instanceof IAction) {
							manager.add((IAction) a);
						} else if (a instanceof IContributionItem) {
							manager.add((IContributionItem) a);
						}
					}
				}
			}
		}

		return manager;
	}

	/**
	 * F&uuml;gt Versorgungsdaten zum Versorgungsdatenfenster hinzu. Das
	 * Versorgungsdatenfenster wird bei Bedarf ge&ouml;ffnet.
	 * 
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 */
	public void addVersorgungsDaten(final AttributeGroup atg, final Aspect asp) {
		if (versorgungFigure == null) {
			openVersorgungWindow();
		}

		versorgungFigure.addVersorgungsDaten(atg, asp);
	}

	/**
	 * Setzt die Sichtbarkeit des Editparts.
	 * 
	 * @param sichtbar
	 *            true, wenn sichtbar
	 */
	public void setSichtbar(final boolean sichtbar) {
		getFigure().setVisible(false);
		if (sourceConnections != null) {
			for (Object ep : sourceConnections) {
				final AnlagenStatusConnectionEditPart acp = (AnlagenStatusConnectionEditPart) ep;

				final EditPart targetEditPart = acp.getTarget();
				((AnlagenStatusEditPart) targetEditPart).setSichtbar(false);
				acp.getFigure().setVisible(false);
			}
		}

		if (targetConnections != null) {
			for (Object ep : targetConnections) {
				final AnlagenStatusConnectionEditPart acp = (AnlagenStatusConnectionEditPart) ep;
				acp.getFigure().setVisible(false);
			}
		}

		setSelected(SELECTED_NONE);
	}

	/**
	 * Bestimmt, ob eine ATG/ASP-Kombination bereits im Versorgungsfenster
	 * angezeigt wird.
	 * 
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @return true, wenn bereits angezeigt, sonst false
	 */
	public boolean isInVersorgung(final AttributeGroup atg, final Aspect asp) {
		if (versorgungFigure == null) {
			return false;
		}

		return versorgungFigure.isInVersorgung(atg, asp);
	}

	/**
	 * Bildet den Status des Edit-Parts.
	 * 
	 * <p>
	 * Der Status ist wie folgt definiert:
	 * </p>
	 * <table>
	 * <tr>
	 * <td>Status</td>
	 * <td>Bedingung</td>
	 * </tr>
	 * <tr>
	 * <td>undefiniert</td>
	 * <td>Der Zustand konnte nicht bestimmt werden</td>
	 * </tr>
	 * <tr>
	 * <td>gest&ouml;rt</td>
	 * <td>Alle untergeordneten Teile sind gest&ouml;rt</td>
	 * </tr>
	 * <tr>
	 * <td>teilgest&ouml;rt</td>
	 * <td>Mindestens eines der untergeordneten Teile ist gest&ouml;rt</td>
	 * </tr>
	 * <tr>
	 * <td>ungest&ouml;rt</td>
	 * <td>Alle untergeordneten Teile sind ungest&ouml;rt</td>
	 * </tr>
	 * </table>
	 * 
	 * @param gesamtAnzahl
	 *            Anzahl der untergeordneten Teile
	 * @param okAnzahl
	 *            Anzahl der untergeordneten Teile mit Status OK
	 * @param teilgestoertAnzahl
	 *            Anzahl der untergeordneten Teile mit Status TEILGESTOERT
	 * @param gestoertAnzahl
	 *            Anzahl der untergeordneten Teile mit Status GESTOERT
	 * @return AnlagenStatusKnotenStatus
	 */
	protected AnlagenStatusKnotenStatus bildeStatus(final int gesamtAnzahl,
			final int okAnzahl, final int teilgestoertAnzahl,
			final int gestoertAnzahl) {

		if (okAnzahl == 0 && teilgestoertAnzahl == 0 && gestoertAnzahl == 0) {
			return AnlagenStatusKnotenStatus.UNBEKANNT;
		}

		if (okAnzahl == gesamtAnzahl) {
			return AnlagenStatusKnotenStatus.OK;
		}

		if (gestoertAnzahl == gesamtAnzahl) {
			return AnlagenStatusKnotenStatus.GESTOERT;
		}

		return AnlagenStatusKnotenStatus.TEILGESTOERT;
	}

	/**
	 * &Ouml;ffnet das Statusfenster.
	 */
	public void zeigeStatusDaten() {
		if (statusFigure == null) {
			openStatusWindow();
		}
	}

	/**
	 * Gibt die Position des Statusfensters zur&uuml;ck.
	 * 
	 * @return Position des Statusfensters
	 */
	public Point getDatenFensterLocation() {
		final Point location = getFigure().getBounds().getLocation();

		final Point loc = new Point();

		loc.x = location.x;
		loc.y = location.y + getFigure().getBounds().getSize().height;

		return loc;
	}

	/**
	 * &Ouml;ffnet das Fenster mit den Statusinformationen.
	 */
	public void openStatusWindow() {
		setStatusFigure(erzeugeStatusFigure());
		statusFigure.zoomChanged(getZoomManager().getZoom());
	}

	/** {@inheritDoc} */
	@Override
	public void zoomChanged(final double zoom) {
		if (isInView()) {
			return;
		}

		super.zoomChanged(zoom);
	}

	/**
	 * Erzeugt die Status-Figure.
	 * 
	 * @return StatusFigure
	 */
	protected abstract StatusFigure erzeugeStatusFigure();

	/**
	 * Schlie&szlig;t das Statusfenster.
	 */
	public void closeStatusWindow() {
		if (statusFigure != null) {
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

				public void run() {
					if (statusFigure != null) {
						// final RootEditPart root = getRoot();
						// final AnlagenStatusTopEditPart tep =
						// (AnlagenStatusTopEditPart) root
						// .getContents();
						final IFigure parent2 = statusFigure.getParent();
						parent2.remove(statusFigure);

						// tep.getContentPane().remove(statusFigure);
						statusFigure = null;
						getKnoten().firePropertyChange(
								ZUSATZWINDOW_CLOSED_PROPERTY, null, null);
					}
				}
			});
		}
	}

	/**
	 * Schliesste ein Zusatzfenster.
	 * 
	 * @param datenFenster
	 *            DatenFenster
	 */
	public void closeZusatzWindow(final DatenFenster datenFenster) {
		if (datenFenster == versorgungFigure) {
			closeVersorgungWindow();
		}

		if (datenFenster == statusFigure) {
			closeStatusWindow();
		}
	}

	/** {@inheritDoc} */
	@Override
	protected void geheOnline() {
		partOnline = true;
		if (getKnoten() != null) {
			knoten.addPropertyChangeListener(this);
			getKnoten().geheOnline();
		}
		super.geheOnline();
	}

	/** {@inheritDoc} */
	@Override
	protected void geheOffline() {
		partOnline = false;
		if (getKnoten() != null) {
			getKnoten().geheOffline();
			knoten.removePropertyChangeListener(this);
		}
		super.geheOffline();
	}

	/** {@inheritDoc} */
	@Override
	public List<ILegendeBaustein> getBausteine() {
		if (legendenBausteine == null) {
			legendenBausteine = new ArrayList<ILegendeBaustein>();

			for (AnlagenStatusKnotenStatus status : AnlagenStatusKnotenStatus
					.values()) {
				final Color color = ColorUtilities
						.longToColor(PluginEinstellungen.getInstance()
								.getEinstellungen()
								.getKnotenStatusBorderColor(status));

				if (color != null) {
					final Display display = Display.getDefault();
					final Image image = new Image(display, 16, 16);
					final GC gc = new GC(image);
					// gc.setForeground(color);
					gc.setBackground(color);
					gc.fillRectangle(image.getBounds());
					gc.dispose();
					// imagelist.add(image);
					legendenBausteine.add(new LegendeBaustein(image, status
							.getName()));
				}
			}
		}

		return legendenBausteine;
	}

	/** {@inheritDoc} */
	@Override
	public Image getIcon() {
		// if (image == null) {
		// final MessQuerschnittFigure f = createFigure();
		// f.setHasData(true);
		// image = FigureUtilities2.createImage(f, new Dimension(16, 16));
		// }
		//
		// return image;

		return null;
	}

	/** {@inheritDoc} */
	@Override
	public String getText() {
		return "Status " + getModel().getDoTyp().getName();
	}

	/**
	 * Setzt Position und Groesse.
	 */
	void setPositionUndGroesse() {
		berechenePosition();
		((AnlagenStatusKnoten) getModel()).setSize(getFigure().getSize());
		berechneAnchor(getFigure());
	}

	/**
	 * Berechnet die Postion des Editparts in der Anzeige (View).
	 */
	void berechenePosition() {
		final int level = ((AnlagenStatusKnoten) getModel()).getlevel();

		final AnlagenStatusTopEditPart tep = (AnlagenStatusTopEditPart) getParent();

		final AnlagenStatusEditPart prev = tep.getVorgaenger(this);

		if (prev == this) {
			return;
		}

		final AnlagenStatusModell m = (AnlagenStatusModell) getParent()
				.getModel();

		int x = 0;
		int y = 0;

		if (((AnlagenStatusKnoten) getModel()).isSichtbar()) {
			if (m.isTopDown()) {
				if (prev == null) {
					x = 0;
				} else {
					x = prev.getLocation().x + prev.getSize().width
							+ HORIZONTAL_SPACE;
				}
				y = tep.getTopDownY(level);
			} else {
				if (prev == null) {
					y = 5;
				} else {
					y = prev.getLocation().y + prev.getSize().height
							+ VERTICAL_SPACE;
				}
				x = tep.getLeftRightX(level);
			}

			tep.addEditPart(level, this);
		}

		((AnlagenStatusKnoten) getModel()).setLocation(new Point(x, y));
	}

	/**
	 * Gibt die Position des Editparts zur&uuml;ck.
	 * 
	 * @return Position des Editparts
	 */
	protected Point getLocation() {
		return ((AnlagenStatusKnoten) getModel()).getLocation();
	}

	/**
	 * Gibt die Gr&ouml;sse des Editparts zur&uuml;ck.
	 * 
	 * @return Gr&ouml;sse des Editparts
	 */
	protected Dimension getSize() {
		return ((AnlagenStatusKnoten) getModel()).getSize();
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelSourceConnections() {

		if (isInView) {
			List<AnlagenStatusConnection> sourceConnections;

			sourceConnections = ((AnlagenStatusKnoten) getModel())
					.getSourceConnections();
			return sourceConnections;
		} else {
			return Collections.emptyList();
		}
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelTargetConnections() {
		// System.out.println(this + ".getModelTargetConnections()");
		if (isInView) {
			return ((AnlagenStatusKnoten) getModel()).getTargetConnections();
		} else {
			return Collections.emptyList();
		}
	}

	/**
	 * Gibt den SourceConnectionAnchor zur&uuml;ck.
	 * 
	 * @param iConnection
	 *            ConnectionEditPart
	 * @return ConnectionAnchor
	 */
	public ConnectionAnchor getSourceConnectionAnchor(
			final ConnectionEditPart iConnection) {
		final AnlagenStatusConnection connection = (AnlagenStatusConnection) iConnection
				.getModel();

		ConnectionAnchor connectionAnchor = sourceAnchorMap.get(connection);

		if (connectionAnchor == null) {
			connectionAnchor = new AnlagenStatusConnectionAnchor(figure);
		}

		return connectionAnchor;
	}

	/**
	 * Return the target connection anchor of the given connection.
	 * 
	 * @param iConnection
	 *            ConnectionEditPart
	 * @return ConnectionAnchor
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(
			final ConnectionEditPart iConnection) {
		final AnlagenStatusConnection connection = (AnlagenStatusConnection) iConnection
				.getModel();
		return targetAnchorMap.get(connection);
	}

	/** {@inheritDoc} */
	@Override
	protected ConnectionEditPart createConnection(final Object iModel) {
		AnlagenStatusConnectionEditPart connectPart = (AnlagenStatusConnectionEditPart) getRoot()
				.getViewer().getEditPartRegistry().get(iModel);
		if (connectPart == null) {
			connectPart = new AnlagenStatusConnectionEditPart();
			connectPart.setModel(iModel);
		}

		return connectPart;
	}

	/**
	 * Return the closest source connection anchor to the request's origin
	 * location.
	 * 
	 * @param iRequest
	 *            Request
	 * 
	 * @return ConnectionAnchor
	 **/
	public ConnectionAnchor getSourceConnectionAnchor(final Request iRequest) {
		return getClosestAnchor(iRequest);
	}

	/**
	 * Return the closest target connection anchor to the request's origin
	 * location.
	 * 
	 * @param iRequest
	 *            Request
	 * 
	 * @return ConnectionAnchor
	 **/
	public ConnectionAnchor getTargetConnectionAnchor(final Request iRequest) {
		return getClosestAnchor(iRequest);
	}

	/**
	 * Returns the closest anchor to request's location.
	 * 
	 * @param iRequest
	 *            Request
	 * @return ConnectionAnchor
	 **/
	private ConnectionAnchor getClosestAnchor(final Request iRequest) {
		return null;
	}

	/**
	 * Berechnet die Anchor.
	 * 
	 * @param figure
	 *            Figure
	 */
	protected void berechneAnchor(final IFigure figure) {
		final List<AnlagenStatusConnectionAnchor> sourceanchors = new ArrayList<AnlagenStatusConnectionAnchor>();
		final List<AnlagenStatusConnectionAnchor> targetanchors = new ArrayList<AnlagenStatusConnectionAnchor>();
		sourcePortAnchorMap.clear();
		targetPortAnchorMap.clear();
		targetAnchorMap.clear();
		sourceAnchorMap.clear();

		Dimension maxSourceLabelDimension = new Dimension(0, 0);
		Dimension maxTargetLabelDimension = new Dimension(0, 0);

		final AnlagenStatusModell m = (AnlagenStatusModell) getParent()
				.getModel();

		final List<AnlagenStatusConnection> modelSourceConnections = getModelSourceConnections();
		if (modelSourceConnections != null) {
			AnlagenStatusConnectionAnchor anchor;
			for (AnlagenStatusConnection conn : modelSourceConnections) {
				if (!conn.isSichtbar()) {
					continue;
				}

				if (sourcePortAnchorMap.get(conn.getSourcePort()) != null) {
					anchor = sourcePortAnchorMap.get(conn.getSourcePort());
				} else {

					anchor = new AnlagenStatusConnectionAnchor(figure);
					anchor.setName("SourceAnchor[" + getKnoten().getLabel()
							+ "]" + ", " + conn);
					sourcePortAnchorMap.put(conn.getSourcePort(), anchor);
					sourceanchors.add(anchor);
				}

				sourceAnchorMap.put(conn, anchor);

				// breitesten Label merken
				maxSourceLabelDimension = maxSourceLabelDimension
						.getUnioned(conn.getSourceLabelDimension());
			}
		}

		if (getTargetConnections() != null) {
			AnlagenStatusConnectionAnchor anchor;
			for (AnlagenStatusConnection conn : getModelTargetConnections()) {
				if (targetPortAnchorMap.get(conn.getTargetPort()) != null) {
					anchor = targetPortAnchorMap.get(conn.getTargetPort());
				} else {

					anchor = new AnlagenStatusConnectionAnchor(figure);
					anchor.setName("TargetAnchor[" + getKnoten().getLabel()
							+ "]" + ", " + conn);
					targetPortAnchorMap.put(conn.getTargetPort(), anchor);
					targetanchors.add(anchor);
				}
				targetAnchorMap.put(conn, anchor);

				// hoechsten Label merken
				maxTargetLabelDimension = maxTargetLabelDimension
						.getUnioned(conn.getTargetLabelDimension());
			}
		}

		final Dimension sourceAnchorDim = new Dimension(
				FIRST_ANCHOR_OFFSET * 2, FIRST_ANCHOR_OFFSET * 2);

		if (sourceanchors.size() > 1) {
			// size()-1 da das Label des letzen Anchors nicht unter die Figure
			// soll, und damit die Anchor gleichmig verteilt sind
			if (m.isTopDown()) {
				sourceAnchorDim.width += (sourceanchors.size() - 1)
						* maxSourceLabelDimension.width;
			} else {
				sourceAnchorDim.height += (sourceanchors.size() - 1)
						* maxSourceLabelDimension.height;
			}
		}

		final Dimension targetAnchorDim = new Dimension(
				FIRST_ANCHOR_OFFSET * 2, FIRST_ANCHOR_OFFSET * 2);

		if (targetanchors.size() > 1) {
			if (m.isTopDown()) {
				targetAnchorDim.width += (targetanchors.size() - 1)
						* maxTargetLabelDimension.width;
			} else {
				targetAnchorDim.height += (targetanchors.size() - 1)
						* maxTargetLabelDimension.height;
			}
		}

		final int abreite = Math.max(sourceAnchorDim.width,
				targetAnchorDim.width);
		final int ahoehe = Math.max(sourceAnchorDim.height,
				targetAnchorDim.height);

		final Dimension knotensize = getKnoten().getSize();
		knotensize.width = Math.max(knotensize.width, abreite);
		knotensize.height = Math.max(knotensize.height, ahoehe);

		// Groesse aktualisieren
		getKnoten().setSize(knotensize);

		// Anchor gleichmssig verteilen
		if (sourceanchors.size() > 0) {
			if (sourceanchors.size() == 1) {
				final AnlagenStatusConnectionAnchor anchor = sourceanchors
						.get(0);
				if (m.isTopDown()) {
					anchor.setOffsetH(knotensize.width / 2);
					anchor.setOffsetV(knotensize.height);
				} else {
					anchor.setOffsetV(knotensize.height / 2);
					anchor.setOffsetH(knotensize.width);
				}
			} else {
				final int hinc = (knotensize.width - (2 * FIRST_ANCHOR_OFFSET))
						/ (sourceanchors.size() - 1);
				final int vinc = (knotensize.height - (2 * FIRST_ANCHOR_OFFSET))
						/ (sourceanchors.size() - 1);
				int hoffset = FIRST_ANCHOR_OFFSET;
				int voffset = FIRST_ANCHOR_OFFSET;
				for (AnlagenStatusConnectionAnchor anchor : sourceanchors) {
					if (m.isTopDown()) {
						anchor.setOffsetH(hoffset);
						anchor.setOffsetV(knotensize.height);
						hoffset += hinc;
					} else {
						anchor.setOffsetV(voffset);
						anchor.setOffsetH(knotensize.width);
						voffset += vinc;
					}
				}
			}
		}

		if (targetanchors.size() > 0) {
			if (targetanchors.size() == 1) {
				final AnlagenStatusConnectionAnchor anchor = targetanchors
						.get(0);
				if (m.isTopDown()) {
					anchor.setOffsetH(knotensize.width / 2);
				} else {
					anchor.setOffsetV(knotensize.height / 2);
				}
			} else {
				final int hinc = (knotensize.width - (2 * FIRST_ANCHOR_OFFSET))
						/ (targetanchors.size() - 1);

				int hoffset = FIRST_ANCHOR_OFFSET;
				int voffset = FIRST_ANCHOR_OFFSET;
				for (AnlagenStatusConnectionAnchor anchor : targetanchors) {
					if (m.isTopDown()) {
						anchor.setOffsetH(hoffset);
						hoffset += hinc;
					} else {
						anchor.setOffsetV(voffset);
						voffset += maxTargetLabelDimension.height;
					}

				}
			}
		}
	}

	/**
	 * Setzt die Kindelemente online/offline.
	 * 
	 * @param online
	 *            true wenn online
	 */
	protected void setKinderOnline(final boolean online) {
		for (Object part : getChildren()) {
			if (online) {
				((AnlagenStatusEditPart) part).geheOnline();
			} else {
				((AnlagenStatusEditPart) part).geheOffline();
			}
			((AnlagenStatusEditPart) part).setKinderOnline(online);
		}
	}

	/**
	 * Gibt den HTML-Text des Editparts zur&uuml;ck.
	 * 
	 * @param col
	 *            Spalte
	 * @param cols
	 *            Spalten
	 * @param first
	 *            erste Spalte ja/nein
	 * @return HTML-Text des Editparts
	 */
	public String getHtmlText(final int col, final int cols, final boolean first) {
		final StringBuffer sb = new StringBuffer();
		final SystemObjekt systemobjekt = getModel().getSystemObjekt();

		if (!first) {
			sb.append("</tr>\n<tr>");
			for (int i = 0; i < col; i++) {
				sb.append("<td></td>");
			}
		}
		sb.append("<td>");

		sb.append("<table border=\"5\" bordercolor=\"" + getHtmlBorderColor()
				+ "\" rules=\"none\" >\n");

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, getHtmlBasisInfo())));

		sb.append("\n<tr><td><table cellspacing=\"1\">\n");

		// Bezeichnung/Name
		final String bez = ((Geraet) systemobjekt).getKdGeraet().getDatum()
				.getBezeichnung();
		if (bez != null && bez.length() == 0) {
			sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null,
					HtmlTools.erzeugeHtmlTabelleSpalte(null, "Bezeichnung:")
							+ HtmlTools.erzeugeHtmlTabelleSpalte(null, bez)));
		}

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null,
						resourceBundle.getString("Knotennummer.label"))
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
								((Geraet) systemobjekt).getKdGeraet()
										.getDatum().getKnotenNummer()
										.toString())));

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, "Status:")
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
								getKnotenStatus().getName())));
		sb.append("</table></td></tr>\n");
		sb.append("</table>\n");

		sb.append("</td>\n");

		boolean subfirst = true;
		for (Object o : getSourceConnections()) {
			final AnlagenStatusConnectionEditPart cep = (AnlagenStatusConnectionEditPart) o;
			final AnlagenStatusEditPart ep = (AnlagenStatusEditPart) cep
					.getTarget();
			sb.append(ep.getHtmlText(col + 1, cols, subfirst));
			subfirst = false;
		}

		if (getSourceConnections().size() == 0) {
			for (int r = 0; r < cols - col; r++) {
				sb.append("<td></td>");
			}
		}
		return sb.toString();
	}

	/**
	 * Erzeugt die Basisinformationen als HTML.
	 * 
	 * @return HTML-Text
	 */
	public String getHtmlBasisInfo() {
		final StringBuffer sb = new StringBuffer();
		final SystemObjekt systemobjekt = getModel().getSystemObjekt();

		sb.append("\n<table cellspacing=\"0\">\n");

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null, HtmlTools
				.erzeugeHtmlTabelleSpalte("align=\"center\" colspan=\"2\"",
						"<h3>" + AnlagenGeraet.getTypText(systemobjekt)
								+ "</h3>")));

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, "Name:")
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
								systemobjekt.getName())));

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, "Pid:")
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
								systemobjekt.getPid())));

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null,
						resourceBundle.getString("Typ.label"))
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null, systemobjekt
								.getTyp().getName()
								+ " ("
								+ systemobjekt.getTyp().getPid() + ")")));

		sb.append("</table>\n");

		return sb.toString();
	}

	/**
	 * Gibt die HTML-Rahmenfarbe zur&uuml;ck.
	 * 
	 * @return HTML-Rahmenfarbe
	 */
	protected String getHtmlBorderColor() {
		final Color color = ColorUtilities.longToColor(PluginEinstellungen
				.getInstance().getEinstellungen()
				.getKnotenStatusBorderColor(getKnotenStatus()));

		final String htmlcolor = "#" + Integer.toHexString(color.getRed())
				+ Integer.toHexString(color.getGreen())
				+ Integer.toHexString(color.getBlue());

		return htmlcolor;
	}

	/**
	 * Gibt einen HTML-td Entry zur&uuml;ck.
	 * 
	 * @param text
	 *            Text
	 * 
	 * @return HTML-td Entry mit Text
	 */
	protected String getHtmlTabEntry(final String text) {
		return "<td>" + text + "</td>\n";
	}
}
