/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.tooltips;

import java.util.ResourceBundle;

import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Panel;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenGeraet;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.plugin.dobj.figures.SystemObjectToolTipFigure;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;

/**
 * Allgemeine Tooltipfigur zur zweizeiligen Anzeige von Tooltips fr
 * Systemobjekte des Datenverteilers.
 * 
 * @author BitCtrl Systems GmbH, peuker
 * @version $Id: AnlagenStatusToolTipFigure.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class AnlagenStatusToolTipFigure extends Panel {

	/**
	 * erzeugt eine Instanz der Tooltipfigur.
	 * 
	 * Der Tooltip wird als Panel mit der zwei untereinander angeordneten
	 * Labels, die Name bzw. PID des bergebenen Systemobjekts enthalten,
	 * realisiert.
	 * 
	 * @param systemObjekt
	 *            das Systemobjekt fr den der Tooltip angelegt werden soll
	 * @param part
	 *            {@link AnlagenStatusEditPart}
	 */
	public AnlagenStatusToolTipFigure(final SystemObjekt systemObjekt,
			final AnlagenStatusEditPart part) {

		final ResourceBundle resourceBundle = PluginAnlagenStatus.getDefault()
				.getResourceBundle();

		setBackgroundColor(ColorConstants.tooltipBackground);

		setLayoutManager(new GridLayout(2, false));
		final Font defaultFont = part.getFontRegistry().defaultFont();

		setFont(defaultFont);

		// Titel
		final Label titelLabel = new Label();
		if (systemObjekt != null) {
			titelLabel.setText(AnlagenGeraet.getTypText(systemObjekt));
		} else {
			titelLabel.setText(resourceBundle
					.getString("KeinSystemobjekt.label"));
		}
		titelLabel.setFont(part.getFontRegistry().getBold(
				defaultFont.getFontData()[0].getName()));
		add(titelLabel);

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = SWT.CENTER;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalSpan = 2;
		getLayoutManager().setConstraint(titelLabel, gridData);

		if (systemObjekt != null) {
			// Name
			add(new Label("Name: "));
			add(new Label(systemObjekt.getName()));

			// Pid
			add(new Label("Pid: "));
			add(new Label(systemObjekt.getPid()));

			// Typ
			add(new Label(resourceBundle.getString("Typ.label") + " "));
			add(new Label(systemObjekt.getTyp().getName() + " ("
					+ systemObjekt.getTyp().getPid() + ")"));
		}

		// setLayoutManager(new GridLayout(1, false));
		// if (systemObjekt != null) {
		// add(new Label(systemObjekt.getName()));
		// add(new Label("(PID: " + systemObjekt.getPid() + ")"));
		// }
	}

	/**
	 * Factory-Methode fr das Anlegen einer Info-Popup-Figur.
	 * 
	 * @param part
	 *            Part
	 * 
	 * @return die Figur oder <code>null</code>
	 */
	public static IFigure getTooltipFigure(final AnlagenStatusEditPart part) {
		IFigure result = null;

		final Object object = part.getModel();
		SystemObjekt systemObjekt = null;
		final Object adapter = Platform.getAdapterManager().getAdapter(object,
				BitCtrlDoModel.class);
		if (adapter != null && adapter instanceof BitCtrlDoModel) {
			final BitCtrlDoModel<?> model = (BitCtrlDoModel<?>) adapter;
			systemObjekt = model.getSystemObjekt();
		}

		if (systemObjekt != null) {
			if (systemObjekt instanceof De) {
				result = new DeTooltip((De) systemObjekt, part);
			} else if (systemObjekt instanceof Eak) {
				result = new EakTooltip((Eak) systemObjekt, part);
			} else if (systemObjekt instanceof Geraet) {
				result = new GeraetTooltip((Geraet) systemObjekt, part);
			} else {
				result = new SystemObjectToolTipFigure(
						systemObjekt.getSystemObject());
			}
		} else {
			return new KeinSystemobjektTooltip(part);
		}

		return result;
	}
}
