/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.views;

import java.util.Iterator;
import java.util.List;
import java.util.ResourceBundle;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagenstatus.actions.AnlagenStatusExportierenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPartFactory;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusRootEditPart;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusTopEditPart;
import de.bsvrz.buv.rw.basislib.legende.ILegendenAbschnitt;

/**
 * Viewer zur Anzeige des Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatus.java 26376 2010-09-17 12:27:01Z gieseler $
 */
public class AnlagenStatus extends ViewPart {

	/** Die View-Id. */
	public static final String VIEW_ID = AnlagenStatus.class.getName();

	/** Die Context-Hilfe-ID. */
	private static final String HILFE_ID = PluginAnlagenStatus.PLUGIN_ID + "."
			+ AnlagenStatus.class.getSimpleName();

	/** Der Viewer. */
	private ScrollingGraphicalViewer graphicalViewer;

	/** Ressourcebundle. */
	private ResourceBundle resourceBundle;

	/** Das GEF-Modell. */
	private AnlagenStatusModell modell;

	/** Manhattan-Routing ja/nein. */
	private boolean manhattan = true;

	/** TopDown ja/nein. */
	private boolean topdown = true;

	/** Legende. */
	private AnlagenStatusLegende legende;

	// private ControlContribution saveContribution;

	/** Aktion zum Einblenden weitere Hierarchieebenen. */
	HierarchieEbenenAction einblendenAction;

	/** Aktion zum Ausblenden von Hierarchieebenen. */
	HierarchieEbenenAction ausblendenAction;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(final Composite parent) {
		graphicalViewer = new ScrollingGraphicalViewer();

		final FigureCanvas controlViewer = (FigureCanvas) graphicalViewer
				.createControl(parent);
		final GridData gridData = new GridData(GridData.FILL_BOTH);
		gridData.widthHint = 300;
		controlViewer.setLayoutData(gridData);
		controlViewer.setBackground(Display.getDefault().getSystemColor(
				SWT.COLOR_WHITE));

		final ScalableFreeformRootEditPart root = new AnlagenStatusRootEditPart();
		graphicalViewer.setRootEditPart(root);
		// root.getZoomManager();

		graphicalViewer.setEditDomain(new EditDomain());
		// List zoomLevels = new ArrayList(3);
		// zoomLevels.add(ZoomManager.FIT_ALL);
		// zoomLevels.add(ZoomManager.FIT_WIDTH);
		// zoomLevels.add(ZoomManager.FIT_HEIGHT);
		// root.getZoomManager().setZoomLevelContributions(zoomLevels);

		// IAction zoomIn = new ZoomInAction(root.getZoomManager());
		// IAction zoomOut = new ZoomOutAction(root.getZoomManager());
		// graphicalViewer.setProperty(MouseWheelHandler.KeyGenerator
		// .getKey(SWT.MOD1), MouseWheelZoomHandler.SINGLETON);

		contributeToActionBars();

		graphicalViewer.setEditPartFactory(new AnlagenStatusEditPartFactory());

		modell.setTopDown(isTopdown());
		modell.setManhattanrouting(isManhattan());

		graphicalViewer.setContents(modell);
		graphicalViewer.setContextMenu(new AnlagenStatusContextMenu(
				graphicalViewer));
		getSite().setSelectionProvider(graphicalViewer);

		legende = new AnlagenStatusLegende("AnlagenstatusLegendeID",
				"Anlagenstatus");

		// Online-Hilfe hinzufgen
		PlatformUI.getWorkbench().getHelpSystem()
				.setHelp(parent, AnlagenStatus.HILFE_ID);

	}

	/**
	 * Test, ob Manhattan-Routing benutzt wird.
	 * 
	 * @return true, wenn Manhattan-Routing benutzt wird
	 */
	public boolean isManhattan() {
		return manhattan;
	}

	/**
	 * Setzt ob Manhattan-Routing benutzt wird.
	 * 
	 * @param manhattan
	 *            true, wenn Manhattan-Routing benutzt wird
	 */
	public void setManhattan(boolean manhattan) {
		this.manhattan = manhattan;
	}

	/**
	 * Test, ob Hierarchierichtung von oben nach unten ist.
	 * 
	 * @return true, wenn von oben nach unten ist
	 */
	public boolean isTopdown() {
		return topdown;
	}

	/**
	 * Setzt die Hierarchierichtung.
	 * 
	 * @param topdown
	 *            true: Hierarchie von oben nach unten, false: Hierarchie von
	 *            rechts nach links
	 */
	public void setTopdown(final boolean topdown) {
		this.topdown = topdown;
	}

	/** {@inheritDoc} */
	@Override
	public void init(final IViewSite site) throws PartInitException {
		super.init(site);
		resourceBundle = PluginAnlagenStatus.getDefault().getResourceBundle();
		topdown = PluginEinstellungen.getInstance().getEinstellungen()
				.isTopDown();
		modell = PluginAnlagenStatus.getDefault().getAnlagenModell();
	}

	/** {@inheritDoc} */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(final Class type) {
		// System.out.println("getAdapter(): " + type);
		if (type == ZoomManager.class) {
			return graphicalViewer.getProperty(ZoomManager.class.toString());
		}

		if (type == ILegendenAbschnitt.class) {
			// System.out.println("Abfrage Legende");
			// // Die Abfrage kommt auch noch mal beim Beenden des Rahmenwerkes,
			// // wenn das LegendeComposite schon disposed ist
			// if ((legende.getLegendeComposite() == null)
			// || !legende.getLegendeComposite().isDisposed()) {
			return legende;
			// }
		}

		return super.getAdapter(type);
	}

	/**
	 * Veranlasst alle Aktionen an Mens und Symbolleisten zu verteilen.
	 */
	private void contributeToActionBars() {
		final IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());

		final IToolBarManager toolbar = bars.getToolBarManager();

		toolbar.add(new AnlagenStatusExportierenAktion(graphicalViewer));

		toolbar.add(new RichtungUmschaltenAction());
		toolbar.add(new RoutingUmschaltenAction());

		toolbar.add(new ZoomComboContributionItem(getViewSite().getPage()));
	}

	/**
	 * F&uuml;llt das View-Men&uuml; mit Aktionen.
	 * 
	 * @param manager
	 */
	private void fillLocalPullDown(final IMenuManager manager) {

		einblendenAction = new HierarchieEbenenAction(
				resourceBundle.getString("ShowNextLevel.label"), 1, this);
		manager.add(einblendenAction);

		ausblendenAction = new HierarchieEbenenAction(
				resourceBundle.getString("UnShowbuttomLevel.label"), -1, this);
		manager.add(ausblendenAction);

		einblendenAction.setEnabled(modell.getShowLevel() < modell
				.getMaxLevel());
		ausblendenAction
				.setEnabled(modell.getShowLevel() > AnlagenStatusModell.VRZ_VIZ_LEVEL);
	}

	/**
	 * Aktion zum Ein-/Ausblenden von Hierarchie-Ebenen.
	 */
	private class HierarchieEbenenAction extends Action {

		/** . */
		private final int level;
		/** . */
		private final ViewPart part;

		/**
		 * Der Konstruktor.
		 * 
		 * @param text
		 *            Text
		 * @param level
		 *            Level
		 * @param anlagenStatus
		 */
		public HierarchieEbenenAction(final String text, final int level,
				AnlagenStatus anlagenStatus) {
			super(text);
			this.level = level;
			this.part = anlagenStatus;
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			// TODO: Cursor busy funktioniert hier nicht: Grund:
			// das DefaultTool (SelectionTool) der EditDomain setzt den
			// Cursor bei 'ViewerEntered' bzw. 'ViewerLeved' immer wieder
			// zurck
			part.showBusy(true);
			try {
				modell.setShowLevel(modell.getShowLevel() + level);
				cleanupParts();
				graphicalViewer.setContents(modell);

				((AnlagenStatusRootEditPart) graphicalViewer.getRootEditPart())
						.bildNeuBerechnen();

				einblendenAction.setEnabled(modell.getShowLevel() < modell
						.getMaxLevel());
				ausblendenAction
						.setEnabled(modell.getShowLevel() > AnlagenStatusModell.VRZ_VIZ_LEVEL);
			} catch (final Exception e) {

			} finally {
				part.showBusy(false);
			}
		}
	}

	/**
	 * Aktion zum Umschalten der Hierarchie-Richtung.
	 */
	private class RichtungUmschaltenAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public RichtungUmschaltenAction() {
			updateButton();
		}

		/**
		 * Aktualisiert den Button.
		 */
		private void updateButton() {
			if (topdown) {
				setToolTipText(resourceBundle
						.getString("LeftRightMode.tooltip"));
				setImageDescriptor(PluginAnlagenStatus.getDefault()
						.getImageDescriptor("icons/leftright.png"));
			} else {
				setToolTipText(resourceBundle.getString("TopDownMode.tooltip"));
				setImageDescriptor(PluginAnlagenStatus.getDefault()
						.getImageDescriptor("icons/topdown.png"));
			}
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			setTopdown(!isTopdown());
			updateButton();
			cleanupParts();
			modell.setTopDown(isTopdown());
			graphicalViewer.setContents(modell);
		}
	}

	/**
	 * Aktion zum Umschalten des Routings.
	 */
	private class RoutingUmschaltenAction extends Action {

		/**
		 * Der Konstruktor.
		 */
		public RoutingUmschaltenAction() {
			updateButton();
		}

		/**
		 * Aktualisiert den Button.
		 */
		private void updateButton() {
			// setText(resourceBundle.getString("MeldungViewMenuExpandAll.label"));
			if (manhattan) {
				setToolTipText(resourceBundle.getString("FanRouting.tooltip"));
				setImageDescriptor(PluginAnlagenStatus.getDefault()
						.getImageDescriptor("icons/fanrouting.png"));
			} else {
				setToolTipText(resourceBundle
						.getString("ManhattanRouting.tooltip"));
				setImageDescriptor(PluginAnlagenStatus.getDefault()
						.getImageDescriptor("icons/manhattanrouting.png"));
			}
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			setManhattan(!isManhattan());
			// modell.setTopDown(topdown);
			updateButton();
			modell.setManhattanrouting(isManhattan());
			final RootEditPart root = graphicalViewer.getRootEditPart();
			final AnlagenStatusTopEditPart tep = (AnlagenStatusTopEditPart) root
					.getContents();
			tep.refresh();
		}
	}

	/**
	 * Aktualisiert alle EditParts.
	 * 
	 * @param children
	 */
	private void refreshParts(final List<?> children) {
		for (Iterator<?> i = children.iterator(); i.hasNext();) {
			EditPart p = (AbstractEditPart) i.next();
			p.refresh();
			refreshParts(p.getChildren());
		}
	}

	/**
	 * Aktualisiert den View.
	 */
	public void refreshView() {
		final RootEditPart root = graphicalViewer.getRootEditPart();
		final AnlagenStatusTopEditPart tep = (AnlagenStatusTopEditPart) root
				.getContents();
		tep.refresh();

		refreshParts(graphicalViewer.getRootEditPart().getChildren());
		legende.aktualisieren();
	}

	/**
	 * L&ouml;scht alle EditParts
	 */
	private void cleanupParts() {
		for (Iterator<?> i = graphicalViewer.getRootEditPart().getChildren()
				.iterator(); i.hasNext();) {
			EditPart p = (AbstractEditPart) i.next();
			p.deactivate();
			p.setModel(null);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
	}

}
