/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.views;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.AnlagenStatusFunktionsgruppe;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;

/**
 * Legende-Composite f&uuml;r Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AnlagenStatusLegendeComposite.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class AnlagenStatusLegendeComposite extends Canvas {

	/** Viewer f&uuml; die Statusfarben. */
	private final TableViewer viewerStatusFarben;

	/** Viewer f&uuml; die FG-farben. */
	private final TableViewer viewerFGFarben;

	/**
	 * Gibt den Viewer zu&uuml;ck.
	 * 
	 * @return Viewer
	 */
	public TableViewer getViewer() {
		return viewerStatusFarben;
	}

	/** {@inheritDoc} */
	@Override
	public boolean setFocus() {
		// TODO Auto-generated method stub
		// return super.setFocus();
		return false;
	}

	/** Liste der Images. */
	private final List<Image> imagelist = new ArrayList<Image>();

	/**
	 * Erzeugt ein Composite f&uuml;r die Legende.
	 * 
	 * @param parent
	 *            Parent-Composite
	 * @param style
	 *            Style
	 */
	public AnlagenStatusLegendeComposite(final Composite parent, final int style) {
		super(parent, style);
		setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));

		final GridLayout pageLayout = new GridLayout();
		// pageLayout.numColumns = 2;
		setLayout(pageLayout);
		setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Group group = new Group(this, SWT.SHADOW_ETCHED_IN);
		group.setText("Zustnde:");
		group.setLayout(new GridLayout(1, true));
		group.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL));
		group.setBackground(Display.getDefault()
				.getSystemColor(SWT.COLOR_WHITE));

		viewerStatusFarben = new TableViewer(group, SWT.NO_FOCUS | SWT.BORDER);
		viewerStatusFarben.setContentProvider(new ArrayContentProvider());
		viewerStatusFarben
				.setLabelProvider(new AnlagenStatusLegendeLabelProvider());

		// Selection abschalten
		viewerStatusFarben
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						viewerStatusFarben.getTable().deselectAll();
					}
				});
		viewerStatusFarben.setInput(AnlagenStatusKnotenStatus.values());

		group = new Group(this, SWT.SHADOW_ETCHED_IN);
		group.setText("Funktionsgruppen:");
		group.setLayout(new GridLayout(1, true));
		group.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.FILL_HORIZONTAL));
		group.setBackground(Display.getDefault()
				.getSystemColor(SWT.COLOR_WHITE));

		viewerFGFarben = new TableViewer(group, SWT.NO_FOCUS | SWT.BORDER);
		viewerFGFarben.setContentProvider(new ArrayContentProvider());
		viewerFGFarben
				.setLabelProvider(new AnlagenStatusLegendeLabelProvider());

		// Selection abschalten
		viewerFGFarben
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						viewerFGFarben.getTable().deselectAll();
					}
				});
		viewerFGFarben.setInput(AnlagenStatusFunktionsgruppe.values());
	}

	/**
	 * Label-Provider f&uuml;r die Legende.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 * 
	 */
	class AnlagenStatusLegendeLabelProvider implements ILabelProvider {

		/** {@inheritDoc} */
		@Override
		public Image getImage(final Object element) {
			Color color = null;

			if (element instanceof AnlagenStatusKnotenStatus) {
				color = ColorUtilities.longToColor(PluginEinstellungen
						.getInstance().getEinstellungen()
						.getKnotenStatusBorderColor(
								(AnlagenStatusKnotenStatus) element));
			}
			if (element instanceof AnlagenStatusFunktionsgruppe) {
				color = ColorUtilities.longToColor(PluginEinstellungen
						.getInstance().getEinstellungen().getFGFarbe(
								(AnlagenStatusFunktionsgruppe) element));
			}

			if (color != null) {
				final Display display = Display.getDefault();
				final Image image = new Image(display, 16, 16);
				final GC gc = new GC(image);
				// gc.setForeground(color);
				gc.setBackground(color);
				gc.fillRectangle(image.getBounds());
				gc.dispose();
				imagelist.add(image);
				return image;
			}

			return null;
		}

		/** {@inheritDoc} */
		@Override
		public String getText(final Object element) {
			if (element instanceof AnlagenStatusKnotenStatus) {
				return ((AnlagenStatusKnotenStatus) element).getName();
			}

			if (element instanceof AnlagenStatusFunktionsgruppe) {
				return ((AnlagenStatusFunktionsgruppe) element).getName();
			}

			return null;
		}

		/** {@inheritDoc} */
		@Override
		public void addListener(final ILabelProviderListener listener) {
			// TODO Auto-generated method stub

		}

		/** {@inheritDoc} */
		@Override
		public void dispose() {
			for (Image i : imagelist) {
				i.dispose();
			}
		}

		/** {@inheritDoc} */
		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			// TODO Auto-generated method stub
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public void removeListener(final ILabelProviderListener listener) {
			// TODO Auto-generated method stub

		}
	}
}
